@@ -1,4 +1,4 @@
-# Copyright 2013-2020 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2013-2022 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -74,7 +74,33 @@ class UserSessionProtocol(object, metaclass=ABCMeta):
            Called at the beginning of a request or after an abort during the request (which may require reinitialisation
            of the session so it may still be used after the abort).
         """
-        
+
+    @classmethod
+    @abstractmethod
+    def preserve_session(cls, session):
+        """
+        Ensure the given session object is kept intact so that it can survive a transaction abort.
+
+        .. versionadded:: 5.2
+        """
+
+    @classmethod
+    @abstractmethod
+    def restore_session(cls, session):
+        """
+        After a transaction is aborted, this ensures the given session is restored to its state before the abort.
+
+        .. versionadded:: 5.2
+        """
+
+    @abstractmethod
+    def get_csrf_token(self):
+        """
+        Fetch/create a suitable token for protecting against CSRF.
+
+        .. versionadded:: 5.2
+        """
+
 
 class SessionDataProtocol(object, metaclass=ABCMeta):
     @classmethod
@@ -85,7 +111,7 @@ class SessionDataProtocol(object, metaclass=ABCMeta):
     @classmethod
     @abstractmethod
     def clear_for_view(cls, view):
-        """Removes all data saved in the scope of the given :class:`reahl.web.ui.UrlBoundView`.
+        """Removes all data saved in the scope of the given :class:`reahl.web.fw.UrlBoundView`.
 
         .. versionadded:: 5.0        
         """
@@ -127,7 +153,7 @@ class UserInputProtocol(SessionDataProtocol, metaclass=ABCMeta):
     @classmethod
     @abstractmethod
     def get_persisted_for_view(cls, view, key, value_type):
-        """Returns the value associated with the given :class:`reahl.web.ui.UrlBoundView`, as previously saved using
+        """Returns the value associated with the given :class:`reahl.web.fw.UrlBoundView`, as previously saved using
            `key`.
 
            .. versionadded:: 5.0
@@ -136,7 +162,7 @@ class UserInputProtocol(SessionDataProtocol, metaclass=ABCMeta):
     @classmethod
     @abstractmethod
     def add_persisted_for_view(cls, view, key, value, value_type):
-        """Persists `value` associated with the given :class:`reahl.web.ui.UrlBoundView`,
+        """Persists `value` associated with the given :class:`reahl.web.fw.UrlBoundView`,
            to be retrieved using `key`.
 
            .. versionadded:: 5.0
@@ -145,7 +171,7 @@ class UserInputProtocol(SessionDataProtocol, metaclass=ABCMeta):
     @classmethod
     @abstractmethod
     def remove_persisted_for_view(cls, view, key):
-        """Removes previously persisted value with the given :class:`reahl.web.ui.UrlBoundView`, as previously saved using
+        """Removes previously persisted value with the given :class:`reahl.web.fw.UrlBoundView`, as previously saved using
            `key`
 
            .. versionadded:: 5.0
