@@ -1,4 +1,4 @@
-# Copyright 2015-2021 Reahl Software Services (Pty) Ltd. All rights reserved.
+# Copyright 2015-2022 Reahl Software Services (Pty) Ltd. All rights reserved.
 #
 #    This file is part of Reahl.
 #
@@ -38,6 +38,8 @@ creating a :class:`Library` and configure your site to use it.
 import itertools
 from collections import OrderedDict
 
+from reahl.component.context import ExecutionContext
+from reahl.component.exceptions import ProgrammerError
 from reahl.web.fw import PackagedFile, ConcatenatedFile
 
 
@@ -69,6 +71,14 @@ class LibraryIndex:
         self.libraries_by_name[library.name] = library
         return library
 
+    def get(self, library_class):
+        matching_libraries = [i for i in self.libraries_by_name.values() if isinstance(i, library_class)]
+        if not matching_libraries:
+            raise ProgrammerError('No library "%s" found in the config.web.frontend_libraries' % library_class)
+        elif len(matching_libraries) > 1:
+            raise ProgrammerError('More than "%s" found in the config.web.frontend_libraries' % library_class)
+        return matching_libraries[0]
+
     def __contains__(self, name):
         """An implementation of the `in` operator, so that one can ask whether a library with given name is in this index.
 
@@ -88,6 +98,8 @@ class LibraryIndex:
         for library in new_libraries:
             self.add(library)
 
+    def __str__(self):
+        return  '%s(%s)' % (self.__class__.__name__, ','.join(self.libraries_by_name.keys()))
 
 class Library:
     """A frontend-library: a collection of CSS and JavaScript code that can be used with Reahl.
@@ -103,6 +115,11 @@ class Library:
 
     :param name: A unique name for this Library.
     """
+    active = True
+    @classmethod
+    def get_instance(cls):
+        return ExecutionContext.get_context().config.web.frontend_libraries.get(cls)
+
     def __init__(self, name):
         self.name = name  #: The unique name of this Library
         self.egg_name = 'reahl-web'  #: The component (egg) that contains the files of this library
@@ -128,26 +145,32 @@ class Library:
             exposed_files.append(ConcatenatedFile('%s.css' % self.name, css_files_to_include))
         return exposed_files
 
-
     def files_of_type(self, extension):
         return [f for f in self.files if f.endswith(extension)]
 
     def header_only_material(self, rendered_page):
         result = ''
-        for file_name in self.files_of_type('.css'):
-            result += '\n<link rel="stylesheet" href="/static/%s" type="text/css">' % file_name
+        if self.active:
+            for file_name in self.files_of_type('.css'):
+                result += '\n<link rel="stylesheet" href="/static/%s" type="text/css">' % file_name
         return result
 
     def footer_only_material(self, rendered_page):
+        result = ''
+        if self.active:
+            for file_name in self.files_of_type('.js'):
+                result += '\n<script type="text/javascript" src="/static/%s"></script>' % file_name
+        return result
+
+    def inline_material(self):
         result = ''
         for file_name in self.files_of_type('.js'):
             result += '\n<script type="text/javascript" src="/static/%s"></script>' % file_name
         return result
 
-
     
 class JQuery(Library):
-    """Version 3.5.1 of `JQuery <https://jquery.com>`_.
+    """Version 3.6.0 of `JQuery <https://jquery.com>`_.
 
     This Library also includes a number of plugins we use internally:
 
@@ -162,8 +185,8 @@ class JQuery(Library):
     """
     def __init__(self):
         super().__init__('jquery')
-        self.files = ['jquery-3.5.1/jquery-3.5.1.js',
-                      'jquery-3.5.1/jquery-3.5.1.min.map']
+        self.files = ['jquery-3.6.0/jquery-3.6.0.js',
+                      'jquery-3.6.0/jquery-3.6.0.min.map']
         self.shipped_in_directory = 'reahl/web/static'
         for i in ['jquery.validate-1.19.3.modified.js',
                   'jquery.ba-bbq-1.3pre.js',
@@ -195,14 +218,14 @@ class JQuery(Library):
 
 
 class JQueryUI(Library):
-    """A heavily customised subset of version 1.12.1 of `JQuery UI <https://jqueryui.com>`_.
+    """A heavily customised subset of version 1.13.1 of `JQuery UI <https://jqueryui.com>`_.
     
    Only contains the `Widget Factory <http://api.jqueryui.com/jQuery.widget/>`_ and :tabbable, :focusable Selector.
     """
     def __init__(self):
         super().__init__('jqueryui')
         self.shipped_in_directory = 'reahl/web/static'
-        self.files = ['jquery-ui-1.12.1.custom/jquery-ui.js']
+        self.files = ['jquery-ui-1.13.1.custom/jquery-ui.js']
 
 
 class HTML5Shiv(Library):
@@ -245,12 +268,14 @@ class Reahl(Library):
     def __init__(self):
         super().__init__('reahl')
         self.shipped_in_directory = 'reahl/web/static'
-        self.files = ['reahl.hashchange.js',
+        self.files = ['reahl.csrf.js',
+                      'reahl.hashchange.js',
                       'reahl.ajaxlink.js',
                       'reahl.primitiveinput.js',
                       'reahl.textinput.js',
                       'reahl.validate.js',
                       'reahl.form.js',
+                      'reahl.plotlychart.js',
                       'reahl.css',
                       'reahl.runningonbadge.css',
                       'runningon.png'
@@ -267,20 +292,20 @@ class Holder(Library):
 
 
 class Bootstrap4(Library):
-    """Version 4.5.3 of `Bootstrap <http://getbootstrap.com/>`_.
+    """Version 4.6.1 of `Bootstrap <http://getbootstrap.com/>`_.
     """
     def __init__(self):
         super().__init__('bootstrap4')
         self.shipped_in_directory = 'reahl/web/static'
         self.files = [
-                      'bootstrap-4.5.3/css/bootstrap.css',
-                      'bootstrap-4.5.3/css/reahl-patch.css',
-                      'bootstrap-4.5.3/css/bootstrap.css.map',
-                      # 'bootstrap-4.5.3/css/bootstrap-grid.css',
-                      # 'bootstrap-4.5.3/css/bootstrap-grid.css.map',
-                      # 'bootstrap-4.5.3/css/bootstrap-reboot.css',
-                      # 'bootstrap-4.5.3/css/bootstrap-reboot.css.map',
-                      'bootstrap-4.5.3/js/bootstrap.js'
+                      'bootstrap-4.6.1/css/bootstrap.css',
+                      'bootstrap-4.6.1/css/reahl-patch.css',
+                      'bootstrap-4.6.1/css/bootstrap.css.map',
+                      # 'bootstrap-4.6.1/css/bootstrap-grid.css',
+                      # 'bootstrap-4.6.1/css/bootstrap-grid.css.map',
+                      # 'bootstrap-4.6.1/css/bootstrap-reboot.css',
+                      # 'bootstrap-4.6.1/css/bootstrap-reboot.css.map',
+                      'bootstrap-4.6.1/js/bootstrap.js'
                       ]
 
 
@@ -322,13 +347,13 @@ class Popper(Library):
 
 
 class Underscore(Library):
-    """Version 1.13.1 of `Underscore.js <https://underscorejs.org>`_.
+    """Version 1.13.2 of `Underscore.js <https://underscorejs.org>`_.
     """
     def __init__(self):
         super().__init__('underscore')
         self.shipped_in_directory = 'reahl/web/static'
         self.files = [
-            'underscore-umd-min.1.13.1.js'
+            'underscore-umd-min.1.13.2.js'
         ]
     def footer_only_material(self, rendered_page):
         return super().footer_only_material(rendered_page) + '<script>var underscore = _;</script>'
@@ -343,3 +368,16 @@ class JsCookie(Library):
         self.files = [
             'js-cookie-2.2.1/js.cookie.js' #this is the UMD version
         ]
+
+
+class PlotlyJS(Library):
+    """Version 2.9.0 of `plotly.js <https://github.com/plotly/plotly.js/>`_.
+    """
+    javascript_filename = 'plotly-2.9.0.min.js'
+    def __init__(self):
+        self.active = False
+        super().__init__('plotly.js')
+        self.shipped_in_directory = 'reahl/web/static'
+        self.files = [
+            self.javascript_filename
+        ]
